package findinfiles;

import java.io.File;
import java.io.FileReader;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.FileNotFoundException;

class Finder {
    
    //Olvassuk be a megadott fajl tartalmat es irjuk ki a konzolra
    static void printFile(String filePath) throws IOException {
        BufferedReader reader = new BufferedReader(new FileReader(new File(filePath)));
        
        for (String line = reader.readLine(); line != null; line = reader.readLine()) {
            System.out.println(line);
        }
    }
    
    //Irjuk ki, ha a file valamelyik sora teljesen megegyezik a megadott szoveggel
    static boolean findLine(String lineToFind, String filePath) throws IOException {
        int lineNum = 0;
        BufferedReader reader = new BufferedReader(new FileReader(new File(filePath)));
        
        for (String line = reader.readLine(); line != null; line = reader.readLine()) {
            lineNum++;
            if (line.equals(lineToFind)) {
                System.out.println("A keresett kifejezes a " + lineNum + ". sorral egyezik meg.");
                return true;
            }
        }
        
        return false;
    }
    
    //Irjuk ki, ha a file valamelyik sora tartalmazza a megadott szoveget
    static boolean findText(String textToFind, String filePath) throws IOException {
        int lineNum = 0;
        BufferedReader reader = new BufferedReader(new FileReader(new File(filePath)));
        
        for (String line = reader.readLine(); line != null; line = reader.readLine()) {
            lineNum++;
            if (line.contains(textToFind)) {
                System.out.println("A keresett kifejezes a " + lineNum + ". sorban talalhato.");
                return true;
            }
        }
        
        return false;
    }
    
    //Irjuk ki, ha a file valamelyik sora tartalmazza a megadott szoveget (nagybetu-erzeketlen valtozat)
    static boolean findTextCaseInsensitive(String textToFind, String filePath) throws IOException {
        textToFind = textToFind.toLowerCase();
        
        int lineNum = 0;
        BufferedReader reader = new BufferedReader(new FileReader(new File(filePath)));
        
        for (String line = reader.readLine(); line != null; line = reader.readLine()) {
            lineNum++;
            if (line.toLowerCase().contains(textToFind)) {
                System.out.println("A keresett kifejezes a " + lineNum + ". sorban talalhato (nagybetu-erzeketlen valtozat).");
                return true;
            }
        }
        
        return false;
    }
    
    //Irjuk ki, ha a file valamelyik sora tartalmazza a megadott szot (nagybetu-erzeketlen valtozat)
    static boolean findWord(String wordToFind, String filePath) throws IOException {
        int lineNum = 0;
        BufferedReader reader = new BufferedReader(new FileReader(new File(filePath)));
        
        String[] words;
        
        for (String line = reader.readLine(); line != null; line = reader.readLine()) {
            lineNum++;
            
            words = line.split("\\W+");     //magyar valtozat: [^a-zA-Z]+
            for(String word : words){
                if (word.equals(wordToFind)) {
                    System.out.println("A keresett szo a " + lineNum + ". sorban talalhato.");
                    return true;
                }
            }
        }
        
        return false;
    }
    
    public static void main(String[] args) throws Exception {
        String fileName = args[0];
        
        System.out.println("Add meg a keresendo szoveget!");
        String text = System.console().readLine();
        
        try {
            System.out.println();
            printFile(fileName);
            System.out.println();
            
            findLine(text, fileName);
            findText(text, fileName);
            findTextCaseInsensitive(text, fileName);
            findWord(text, fileName);
        } catch(FileNotFoundException e) {
            System.err.println("A(z) " + fileName + " fajl nem talalhato.");
        } catch(IOException e) {
            System.err.println("Olvasasi hiba.");
        }
    }
}

