import java.util.Arrays;

public class IntTree {
	public final int value;

	private int size;
	private IntTree left;
	private IntTree right;

	public IntTree(int x) {
		value = x;
		size  = 1;
		left  = null;
		right = null;
	}
	
	public static IntTree make(int[] xs) {
		if (xs.length == 0) return null;
		IntTree result = new IntTree(xs[0]);
		for (int i = 1; i < xs.length; i++) result.insert(xs[i]);
		return result;
	}
	
	public void insert(int x) {
		if (x < value) {
			if (left != null) left.insert(x);
			else left = new IntTree(x);
		}
		else { // ">="
			if (right != null) right.insert(x);
			else right = new IntTree(x);
		}
		this.size += 1;
	}
	
	public boolean contains(int x) {
		return
		  (x == value) ||
		  ((x < value) && (left != null) && left.contains(x)) ||
		  ((x >= value) && (right != null) && right.contains(x));
	}
	
	public int size() {
		// return ((left != null) ? left.size() : 0) + 1 + ((right != null) ? right.size() : 0);
		return this.size;
	}
	
	public int[] toArray() {
		int[] result = new int[size()];
		int[] leftElements  = (left != null) ? left.toArray() : new int[] {};
		int[] rightElements = (right != null) ? right.toArray() : new int[] {};
		int i, j;
		for (i = 0; i < leftElements.length; i++) result[i] = leftElements[i];
		result[i++] = value;
		for (j = 0; j < rightElements.length; i++, j++) result[i] = rightElements[j];
		return result;
	}
	
	public boolean equalsTo(IntTree t) {
		return Arrays.equals(toArray(), t.toArray());
	}

	public boolean equalsTo2(IntTree t) {
        return (value == t.value) &&
		  (((left != null) && left.equalsTo2(t.left)) || left == null) &&
		  (((right != null) && right.equalsTo2(t.right)) || right == null);
	}
}
