package utils;

public class IntList {
	private int[] store;
	private int   storeSize;
	private static final int ALLOCATION_UNIT = 32;

	public int[] toArray() {
		int[] result = new int[storeSize];
		for (int i = 0; i < storeSize; i++) {
			result[i] = store[i];
		}
		return result;
	}

	private void resize(int sizeToAllocate) {
		int[] newStore = new int[sizeToAllocate];
		for (int i = 0; i < store.length; i++) {
			newStore[i] = store[i];
		}
		store = newStore;
	}

	public void add(int i, int x) {
		if (i < 0) return;
		if (i >= store.length) {
			resize(((i / ALLOCATION_UNIT) + 1) * ALLOCATION_UNIT);
			storeSize = i + 1;
		}
		else {
			storeSize += 1;
		}
		for (int j = storeSize - 1; i < j; j--) {
			store[j] = store[j - 1];
		}
		store[i] = x;
	}

	public void add(int x) {
		if (storeSize >= store.length) resize(store.length + ALLOCATION_UNIT);
		store[storeSize] = x;
		storeSize += 1;
	}
	
	private boolean isValidIndex(int n) {
		return (0 <= n && n < storeSize);
	}
	
	public Integer get(int i) {
		return isValidIndex(i) ? store[i] : null;
	}

	public void set(int i, int x) {
		if (isValidIndex(i)) store[i] = x;
	}
	
	public IntList() {
		store = new int[ALLOCATION_UNIT];
		storeSize = 0;
	}

	public IntList(int[] input) {
		int sizeToAllocate = ((input.length / ALLOCATION_UNIT) + 1) * ALLOCATION_UNIT;
		store = new int[sizeToAllocate];
		for (int i = 0; i < input.length; i++) {
			store[i] = input[i];
		}
		storeSize = input.length;
	}

	public void clear() {
		storeSize = 0;
	}

	public int size() {
		return storeSize;
	}
}
